
/*******************************************************}
{                                                       }
{       FuturaSoft TAP Library                          }
{       Volume Control                                  }
{       Version 1.00 - 27. December 2005                }
{       For Topfield 5200PVRc                           }
{                                                       }
{       Copyright (c) 2005-2006 Martin Krmer           }
{                                                       }
{       E-mail: tap10~futurasoftde                     }
{                                                       }
{*******************************************************/

#include "tap.h"

#define ID_Volume 0x97010001

TAP_ID(ID_Volume);
TAP_PROGRAM_NAME("VolumeControl");
TAP_AUTHOR_NAME("FuturaSoft");
TAP_DESCRIPTION("Volume Control");
TAP_ETCINFO(__DATE__);

#define ScreenWidth  720
#define ScreenHeight 576

#define cmdShow 1
#define cmdHide 2
#define cmdExit 4

static int VolumeCommand = 0;
static bool VolumeLocked = FALSE;
static bool VolumeVisible = FALSE;

static word VolumeMemRegion = 0;
static word VolumeRegion = 0;

static int VolumeLeft = 540;
static int VolumeTop = 40;
static int VolumeWidth = 0;
static int VolumeHeight = 26;

static dword VolumeDisplayTime = 400;

static byte LastSoundLevel = 255;
static dword LastVolumeTick = 0;


void DrawVolume(word Region, byte SoundLevel)
{
  char Text[128];
  dword TextColor;

  TAP_SPrint(Text, "Volume: %d", SoundLevel & 0x7F);

  TextColor = COLOR_Yellow;
  if ( SoundLevel & 0x80 )
    TextColor = COLOR_Red;

  TAP_Osd_FillBox(VolumeMemRegion, 0, 0, VolumeWidth, VolumeHeight, COLOR_Black);
  TAP_Osd_PutS(VolumeMemRegion, 4, 0, VolumeWidth - 1, Text, TextColor, COLOR_None, 0, FNT_Size_1926, 0, ALIGN_LEFT);

  TAP_Osd_Copy(VolumeMemRegion, Region, 0, 0, VolumeWidth, VolumeHeight, 0, 0, FALSE);

  LastSoundLevel = SoundLevel;
  LastVolumeTick = TAP_GetTick();
}

void RefreshVolume(void)
{
  byte Level;

  Level = TAP_GetSystemVar(SYSVAR_Vol);

  if ( Level != LastSoundLevel )
    DrawVolume(VolumeRegion, Level);
}

void ShowVolume(void)
{
  if ( VolumeVisible )
    return;

  //TAP_ExitNormal();
  VolumeMemRegion = TAP_Osd_Create(VolumeLeft, VolumeTop, VolumeWidth, VolumeHeight, 0, OSD_Flag_MemRgn);
  VolumeRegion = TAP_Osd_Create(VolumeLeft, VolumeTop, VolumeWidth, VolumeHeight, 0, 0);
  DrawVolume(VolumeRegion, TAP_GetSystemVar(SYSVAR_Vol));

  VolumeVisible = TRUE;
}

void HideVolume(void)
{
  if ( !VolumeVisible )
    return;

  VolumeVisible = FALSE;

  TAP_Osd_Delete(VolumeMemRegion);
  TAP_Osd_Delete(VolumeRegion);
  //TAP_EnterNormal();
}


dword TAP_EventHandler(word event, dword param1, dword param2)
{
  dword mainState;
  dword subState;
  dword DisplayTime;

  TAP_GetState(&mainState, &subState);

  if ( (mainState != STATE_Normal /*|| subState != SUBSTATE_Normal*/ )
    && (mainState != STATE_Tap || subState != ID_Volume) )
    return param1;

  if ( event == EVT_IDLE )
  {
    if ( !VolumeLocked )
    {
      VolumeLocked = TRUE;

      if ( VolumeCommand )
      {
        if ( VolumeCommand & cmdExit )
        {
          HideVolume();
          TAP_Exit();
        }
        else if ( VolumeCommand & cmdHide )
          HideVolume();
        else if ( VolumeCommand & cmdShow )
          ShowVolume();

        VolumeCommand = 0;
      }

      if ( VolumeVisible )
      {
        DisplayTime = TAP_GetTick() - LastVolumeTick;
        if ( DisplayTime > VolumeDisplayTime )
          HideVolume();
        else
          RefreshVolume();
      }

      VolumeLocked = FALSE;
    }

    return param1;
  }

  if ( event == EVT_KEY )
  {
    if ( !VolumeVisible )
      switch ( param1 )
      {
        case RKEY_Uhf :
          VolumeCommand = (VolumeCommand | cmdShow) & ~cmdHide;
          return 0;

        case RKEY_VolUp :
        case RKEY_VolDown :
          VolumeCommand = (VolumeCommand | cmdShow) & ~cmdHide;
          return param1;
      }
    else
      switch ( param1 )
      {
        case RKEY_Uhf :
          VolumeCommand = (VolumeCommand | cmdHide) & ~cmdShow;
          return 0;

        case RKEY_Exit :
          VolumeCommand = cmdExit;
          return 0;
      }
  }

  return param1;
}

int TAP_Main(void)
{
  char Text[128];

  TAP_SPrint(Text, "Volume: %d", 99);
  VolumeWidth = TAP_Osd_GetW(Text, 0, FNT_Size_1926) + 8;

  return 1;
}
